@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 2015-2022 The Khronos Group Inc.
- * Copyright (c) 2015-2023 Valve Corporation
- * Copyright (c) 2015-2023 LunarG, Inc.
+ * Copyright (c) 2015-2024 Valve Corporation
+ * Copyright (c) 2015-2024 LunarG, Inc.
  * Copyright (c) 2015-2022 Google, Inc.
  * Copyright (c) 2015-2023 Nvidia Corporation.
  *
@@ -28,10 +28,6 @@
 #include "extension_layer_tests.h"
 #include <vulkan/utility/vk_struct_helper.hpp>
 
-#if !defined(ANDROID)
-#include "test_layer_location.h"
-#endif
-
 // Global list of sType,size identifiers
 std::vector<std::pair<uint32_t, uint32_t>> custom_stype_info{};
 
@@ -229,7 +225,7 @@ bool VkExtensionLayerTest::CheckDecompressionSupportAndInitState() {
     return true;
 }
 
-bool VkExtensionLayerTest::CheckShaderObjectSupportAndInitState() {
+bool VkExtensionLayerTest::CheckShaderObjectSupportAndInitState(bool meshShaders) {
     if (!DeviceExtensionSupported(VK_EXT_SHADER_OBJECT_EXTENSION_NAME, 0)) {
         return false;
     }
@@ -261,6 +257,12 @@ bool VkExtensionLayerTest::CheckShaderObjectSupportAndInitState() {
     }
 
     vkGetPhysicalDeviceFeatures2(gpu(), &features2);
+    if (!meshShaders) {
+        mesh_shader_features.taskShader = VK_FALSE;
+        mesh_shader_features.meshShader = VK_FALSE;
+    }
+    mesh_shader_features.multiviewMeshShader = VK_FALSE;
+    mesh_shader_features.primitiveFragmentShadingRateMeshShader = VK_FALSE;
     if (!shader_object_features.shaderObject || !dynamic_rendering_features.dynamicRendering) {
         return false;
     }
@@ -630,7 +632,7 @@ int main(int argc, char **argv) {
 #endif
 
     // Set VK_LAYER_PATH so that the loader can find the layers
-    VkTestFramework::setEnvironmentSetting("VK_LAYER_PATH", LAYER_BUILD_LOCATION);
+    VkTestFramework::setEnvironmentSetting("VK_LAYER_PATH", SHADER_OBJECT_BINARY_PATH);
 
     ::testing::InitGoogleTest(&argc, argv);
     VkTestFramework::InitArgs(&argc, argv);
